/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package de.nplusc.izc.tools.IOtools.iZformats;

import de.nplusc.izc.tools.IOtools.FileTK;
import de.nplusc.izc.tools.baseTools.arraytools;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.util.ArrayList;

/**
 *
 * @author LH
 */
public class IZpackage
{
    
    static final byte[] magicHeader = new byte[]{(byte)0x00, (byte)0x14, (byte)0x19, (byte)0x40, (byte)0xFA ,(byte)0xC4 ,(byte)0x0A};
    public static final int CHUNK_SIZE=4096;
    //Formatspezifikation
    //magicHEader=0x00 00 14 19 40 FA C4 0A
    //länge desy eigentlichen headers in byte >in 4 bytes speichern
    //anzahl eintrage; 4 bytes//wichtig beim einlesen
    //Headerformat: 1 byte Länge des pfades; Pfad (0-255 lang; relativ)8 bytes Offset innerhalb Datei; 8 byte Größe
    //Dann dateien in einem Bytestrom aneinandergehängt
    
    private String packageToRead;
    //private boolean archiveIsreadOnly;//TODO: erkennen ob medium Disk ist
    private RandomAccessFile achv=null;
    private String[] header_fnames = null;;
    private long[] header_fsizes=null;
    private long[] header_foffsets=null;
    
    
    
    public IZpackage(String path)
    {
        packageToRead = path;
        try
        {
            //NOP
            achv = new RandomAccessFile(packageToRead, "rwd");//readonly als absicherung gegen derps
            if(FileTK.checkDirExist(path))
            {
                preloadHeader();
            }
        }
        catch (FileNotFoundException ex)
        {
            ex.printStackTrace();
        }
    }
    
   
    
    void preloadHeader()
    {
        try
        {
            achv.seek(8);//headergröße anspringen
            byte[] hr = new byte[4];
            achv.readFully(hr);
            int hs = ByteBuffer.allocate(4).put(hr).getInt(0);

            achv.seek(12);//elementeanzahl anspringen
            byte[] fc = new byte[4];
            achv.readFully(fc);
            int elemcount= ByteBuffer.allocate(4).put(fc).getInt(0);

            achv.seek(16);
            byte[] hdrr = new byte[hs];
            achv.readFully(hdrr);//einlesen des headers
            ByteBuffer hdr = ByteBuffer.allocate(hs).put(hdrr);
            int pos = 0;//position innerhalb headers
            String[] filenames = new String[elemcount];
            long[] filesizes=new long[elemcount],fileoffsets=new long[elemcount];//die 3 arrays initialisieren um den header abzubilden //dateinamenlänge wird nur intern benötigt
            for (int i = 0; i < elemcount; i++)
            {
                int flength = hdr.get(pos)&0xFF;//vorzeichen rausbügeln; danke javaquirk
                pos++;
                byte[] filenameraw = new byte[flength];
                hdr.position(pos);
                hdr.get(filenameraw);
                pos+=flength;
                filenames[i]=new String(filenameraw);//etwas unkonventionell :P dürfte nicht jeder wissen dass das geht
                filesizes[i]=hdr.getLong(pos);
                pos+=8;
                fileoffsets[i]=hdr.getLong(pos);
                pos+=8;
            }
            //header eingelesen; 
            //header dumpen
            header_fnames = filenames;
            header_foffsets = fileoffsets;
            header_fsizes = filesizes;
        }
        catch(IOException e)
        {
        }
    }
    
   
    
    
    
    
    /**
     * erstellt ein neues iZpack-archiv unter dem im Objekt hinterlegtren archiv
     * @param baseDir Wurzelverzeichnis der input-files
     * @param files Dateien; relativ zu basedir; da dies die struktur im archiv spieschelt
     */
    public void createArchive(String baseDir,String... files)
    {
        byte[] filenamelengths = new byte[files.length];
        byte[][] fileBytes = new byte[files.length][];
        byte[][] fileSizes = new byte[files.length][8];
        long[] fileSizesRaw = new long[files.length];//nötig um Offset auszurechnen und einzuspeixern
        byte[][]  offsets = new byte[files.length][8];//wird erst zum schluss befüllt;
        int headerSize=0;
        //oFFSET0ALLE VORHERIGEN OFFSETS + HEADERSIZE+ 8DA MAGISCHE KENNUNG9
        for (int i = 0; i < files.length; i++)
        {
            String fileToGetSize = baseDir+"\\"+files[i];
            
            long fileSize = new File(fileToGetSize).length();
            fileSizesRaw[i] = fileSize;
            //byte[] filesizearr = new byte[8];
            /*for (int j = 0; j < 8; j++)
            {
                filesizearr[7-j] = (byte)(fileSize%256);//schneidet niederwertigstes bit heraus; 
                fileSize/=256;//ein byte nach rechts dudeln;
                
            }*/
            fileSizes[i]=ByteBuffer.allocate(8).putLong(fileSize).array();
            fileBytes[i]=files[i].getBytes();
            filenamelengths[i] = (byte)(fileBytes[i].length);
                         //size;offset   //dateinamenlänge  //längenmarkerbyte
            headerSize += 8+      8+     filenamelengths[i]+1;
        }
        long baseoffset = headerSize+16; //Header+magischeKennung;
        
        //create_offsets
        for (int i = 0; i < fileSizesRaw.length; i++)
        {
           
                offsets[i] = ByteBuffer.allocate(8).putLong(baseoffset).array();
                baseoffset +=fileSizesRaw[i]; //offset für nxte datei vorbereiten;
            
        }
        
        byte[] header;//= new byte[headerSize+4];//headerSizeFlag wird mit eingebettet//elemcount auch;
        byte[] headerSizeattrib = ByteBuffer.allocate(4).putInt(headerSize).array();
        ByteBuffer headerAssem = ByteBuffer.allocate(headerSize+8);
        //header zusammensetzen;
        byte[] headerElemCount = ByteBuffer.allocate(4).putInt(files.length).array();
        int bo = 0;
        headerAssem.put(headerSizeattrib);
        bo+=4;//weiterschieben
        headerAssem.position(bo);
        headerAssem.put(headerElemCount);
        bo+=4;
        headerAssem.position(bo);
        
        for (int i = 0; i < offsets.length; i++)
        {
            headerAssem.put(bo,filenamelengths[i]);
            bo++;
            headerAssem.position(bo);
            headerAssem.put(fileBytes[i]);
            bo+=((int)filenamelengths[i]&0xff);
            headerAssem.position(bo);
            headerAssem.put(fileSizes[i]);
            bo+=8;
            headerAssem.position(bo);
            headerAssem.put(offsets[i]);     
            bo+=8;
            headerAssem.position(bo);
        }
        header=headerAssem.array();
        //RandomAccessFile achv=null;
        try
        {
            achv.seek(0);//zurücksetzen der datei;
            achv.write(magicHeader);
            achv.seek(8);
            achv.write(header);
            for (String fn : files)
            {
                RandomAccessFile readIn = new RandomAccessFile(baseDir+"\\"+fn, "rwd");
                long readInVol = readIn.length();
                while(readIn.getFilePointer()<readIn.length())
                {
                    byte[] buf;
                    if(readInVol>CHUNK_SIZE)
                    {
                    buf = new byte[CHUNK_SIZE];
                    readInVol-=CHUNK_SIZE;
                    }
                    else
                    {
                        buf = new byte[(int)readInVol];
                    }
                    readIn.read(buf, 0, buf.length);  //TODO effizioenteren weg finden!!!
                    achv.write(buf);
                }
                readIn.close();
            }
            
        }
        catch(IOException e)
        {
            
        }
        preloadHeader();
        
    }
    
    
    public void dumpAllIntoDirectory(String path)
    {
        try
        {
        long[] fileoffsets = header_foffsets;
        long[] filesizes = header_fsizes;
        String[] filenames = header_fnames;
        for (int i = 0; i < fileoffsets.length; i++)
            {
                long offs = fileoffsets[i];
                String fn = path+"\\"+filenames[i];
                long fs = filesizes[i];
                achv.seek(offs);
                FileTK.ensuredirExistence(fn);
                RandomAccessFile dump = new RandomAccessFile(fn,"rwd");
                long dist = fs;
                while(achv.getFilePointer()<fs+offs)
                {
                    byte[] buf;
                    if(dist>CHUNK_SIZE)
                    {
                    buf = new byte[CHUNK_SIZE];
                    dist-=CHUNK_SIZE;
                    }
                    else
                    {
                        buf = new byte[(int)dist];
                    }
                    achv.read(buf, 0, buf.length);  //TODO effizioenteren weg finden!!!
                    dump.write(buf);
                    
                    
                    
                    /*
                    byte[] buf = new byte[1];
                    achv.read(buf, 0, 1);  //TODO effizioenteren weg finden!!!
                    dump.write(buf);*/
                }
                dump.close();
            }
        }
        catch(IOException e)
        {
            
        }
    }
    
    public void extractFileToDisk(String achvpath,String destdir)
    {
                try
        {
            int pos = arraytools.getElemPosInArray(header_fnames, achvpath);
            byte[] target = new byte[(int)header_fsizes[pos]];
            achv.seek(header_foffsets[pos]);
            FileTK.ensuredirExistence(destdir+"\\"+achvpath);
            RandomAccessFile dump = new RandomAccessFile(destdir+"\\"+achvpath, "rwd");
            long fs=header_fsizes[pos],readInVol=header_fsizes[pos],offs=header_foffsets[pos];
            while(achv.getFilePointer()<fs+offs)
            {
                byte[] buf;
                if(readInVol>CHUNK_SIZE)
                {
                buf = new byte[CHUNK_SIZE];
                readInVol-=CHUNK_SIZE;
                }
                else
                {
                    buf = new byte[(int)readInVol];
                }
                achv.read(buf, 0, buf.length);  //TODO effizioenteren weg finden!!!
                dump.write(buf);
            }
            dump.close();
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
    }
    
    
    
    
    /**
     * Liest eine Datei in jnen String aus. wird nur für kleine files empfohlen;
     * @param archivefile Datei im Archiv; muss in achvfile sein
     * @return Inhalt als String
     */
    public String getFileContent(String archivefile)
    {
        try
        {
            int pos = arraytools.getElemPosInArray(header_fnames, archivefile);
            byte[] target = new byte[(int)header_fsizes[pos]];
            achv.seek(header_foffsets[pos]);
            achv.read(target);
            return new String(target);
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
        return "";
    }
    
    public String[] getFileList()
    {
        //hauptarbeit wird beim einlesen der datei gemacht;
        return header_fnames;
    }
   
    
    
    
    
    
    /*
    public void AddFile(String pathOnDisk)
    {
        
        
        
        
        throw new UnsupportedOperationException("Muss dies noch machen");
    }
    

    
    
    public void removeFile(String path)
    {
        throw new UnsupportedOperationException("Muss dies noch machen");
    }*/
    
    
    
    
    public void unload()
    {
        try
        {
            achv.close();
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }
    }
    

    
    
    
    
    //################################################################################################################
    //################################################################################################################
    //################################################################################################################
    //################################################################################################################
    //################################################################################################################
    //static utility methods for faster archivehandling
    
    
    public static void ExtractFilesToPath(String archivefile, String path)
    {
        IZpackage temp = new IZpackage(archivefile);
        temp.dumpAllIntoDirectory(path);
        temp.unload();
    }
     
    public static void wrapArchive(String archive,String packdir)
    {
        IZpackage temp = new IZpackage(archive);
        String[] filecontent = FileTK.walkDirectoryRecursively(packdir);
        ArrayList<String> fnp = new ArrayList<>();
        for (int i=0;i<filecontent.length;i++)//String fn : filecontent)
        {
            String fn=filecontent[i];
            
            if(!FileTK.isDir(fn))
            {
                fn=FileTK.getRelativePath(fn, packdir).substring(3);
                fnp.add(fn);
                
            }
        }
        temp.createArchive(packdir, fnp.toArray(new String[0]));
        temp.unload();
    }
    public String getPath()
    {
        return packageToRead;
    }
}
